/*
    lib/comedi.c
    comedi library routines

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>

#define inline 

#include <comedilib.h>

unsigned int comedi_version_code=COMEDI_VERSION_CODE;

static int comedi_sc_isvalid(comedi_t *it,unsigned int subdevice,int chan);

#define COMEDILIB_MAGIC 0xc001dafe


comedi_t *comedi_open(char *fn)
{
	comedi_t *it;

	if(!(it=malloc(sizeof(comedi_t))))
		goto cleanup;

	if((it->fd=open(fn,O_RDWR))<0)
		goto cleanup;

	if(comedi_get_devinfo(it->fd,&it->devinfo)<0)
		goto cleanup;

	it->n_subdevices=it->devinfo.n_subdevs;

	comedi_update_subd_info(it);

	it->magic=COMEDILIB_MAGIC;

	return it;
cleanup:
	if(it)
		free(it);

	return NULL;
}


void comedi_close(comedi_t *it)
{
	it->magic=0;

	close(it->fd);
}


/* ioctl wrappers */

int comedi_get_devinfo(int fd,comedi_devinfo *it)
{
	return ioctl(fd,COMEDI_DEVINFO,it);
}

int comedi_get_subdinfo(int fd,comedi_subdinfo *it)
{
	return ioctl(fd,COMEDI_SUBDINFO,it);
}

int comedi_get_chaninfo(int fd,unsigned int subdev,sampl_t *maxdata_list,unsigned int *flaglist,unsigned int *rangelist)
{
	comedi_chaninfo ci;

	ci.subdev=subdev;
	ci.flaglist=flaglist;
	ci.rangelist=rangelist;
	ci.maxdata_list=maxdata_list;

	return ioctl(fd,COMEDI_CHANINFO,&ci);
}

int comedi_trigger(int fd,comedi_trig *it)
{
	return ioctl(fd,COMEDI_TRIG,it);
}

int comedi_lock(int fd,int subdevice)
{
	return ioctl(fd,COMEDI_LOCK,subdevice);
}

int comedi_unlock(int fd,int subdevice)
{
	return ioctl(fd,COMEDI_UNLOCK,subdevice);
}

int comedi_cancel(int fd,int subdevice)
{
	return ioctl(fd,COMEDI_CANCEL,subdevice);
}


/* ioctl wrapper wrappers */

int comedi_update_subd_info(comedi_t *it)
{
	it->subdevices=realloc(it->subdevices,
			sizeof(comedi_subdinfo)*it->n_subdevices);

	return comedi_get_subdinfo(it->fd,it->subdevices);
}

int comedi_update_chan_info(comedi_t *it,unsigned int subdevice)
{
	/* XXX */

	return -1;
}


/* */

static int comedi_sc_isvalid(comedi_t *it,unsigned int subdevice,int chan)
{
	if(it->magic != COMEDILIB_MAGIC)
		return 0;
	if(subdevice>=it->n_subdevices)
		return 0;
	if(it->subdevices==NULL)
		return 0;
	if(chan>=it->subdevices[subdevice].n_chan)
		return 0;

	return 1;
}

int comedi_get_maxdata(comedi_t *it,unsigned int subdevice,int chan)
{
	if(!comedi_sc_isvalid(it,subdevice,chan))
		return -1;

	if(it->subdevices[subdevice].maxdata)
		return it->subdevices[subdevice].maxdata;

#if 0
	/* XXX */
	return it->subdevices[subdevice].maxdata_list[chan];
#else
	return 0;
#endif
}

int comedi_get_rangetype(comedi_t *it,unsigned int subdevice,int chan)
{
	if(!comedi_sc_isvalid(it,subdevice,chan))
		return -1;

	if(it->subdevices[subdevice].range_type)
		return it->subdevices[subdevice].range_type;

#if 0
	/* XXX */
	return it->subdevices[subdevice].rangetype_list[chan];
#else
	return 0;
#endif
}



/* */

int comedi_put(int fd,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref,sampl_t data)
{
	comedi_trig t;

	chan=CR_PACK(chan,range,aref);

	t.subdev=subdev;
	t.mode=0;
	t.flags=0;
	t.n_chan=1;
	t.chanlist=&chan;
	t.data=&data;
	t.n=1;
	t.trigsrc=0;
	t.trigvar=0;
	t.trigvar1=0;

	return comedi_trigger(fd,&t);
}


int comedi_get(int fd,unsigned int subdev,unsigned int chan,unsigned int range,
		unsigned int aref)
{
	comedi_trig t;
	int ret;
	sampl_t data;

	chan=CR_PACK(chan,range,aref);
	
	t.subdev=subdev;
	t.mode=0;
	t.flags=0;
	t.n_chan=1;
	t.chanlist=&chan;
	t.data=&data;
	t.n=1;
	t.trigsrc=0;
	t.trigvar=0;
	t.trigvar1=0;

	ret=comedi_trigger(fd,&t);
	if(ret<0)
		return ret;

	return data;
}


