/*
    lib/range.c
    comedi library routines for voltage ranges

    COMEDI - Linux Control and Measurement Device Interface
    Copyright (C) 1997-8 David A. Schleef <ds@stm.lbl.gov>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <comedi.h>
#include <string.h>
#include <nan.h>

#include <comedilib.h>

comedi_range comedi_ranges[]={
/* --BEGIN-RANGE-DEFS-- */
/* 	lib/range.def */
/* 	definition file for range information */
/*  */
/* 	COMEDI - Linux Control and Measurement Device Interface */
/* 	Copyright (C) 1998 David A. Schleef <ds@stm.lbl.gov> */
/*  */
/* 	Automatically generated from lib/range.def by scripts/util/mk_range */
/*  */
/*  general form for range.def file: */
/* 	* unique_name_for_this_range */
/* 		first_range_min		first_range_max		unit */
/* 		second_range_min	second_range_max	unit */
/*  etc... */
/*  */
/*  unit is either V, A, ext, or none.  Default is V. */
/*  unit 'ext' refers to external reference */
/*  */
/*  This file is _append only_.  Inserting range definitions breaks */
/*  binary compatibility. */
/* bipolar10 */
	{-10,10},
/* bipolar5 */
	{-5,5},
/* bipolar2_5 */
	{-2.5,2.5},
/* unipolar10 */
	{0,10},
/* unipolar5 */
	{0,5},
/* unipolar2_5 */
	{0,2.5},
/* dt282x_ai_lo_bipolar */
	{-10,10},
	{-5,5},
	{-2.5,2.5},
	{-1.25,1.25},
/* dt282x_ai_lo_unipolar */
	{0,10},
	{0,5},
	{0,2.5},
	{0,1.25},
/* dt282x_ai_5_bipolar */
	{-5,5},
	{-2.5,2.5},
	{-1.25,1.25},
	{-0.625,0.625},
/* dt282x_ai_5_unipolar */
	{0,5},
	{0,2.5},
	{0,1.25},
	{0,0.625},
/* dt282x_ai_hi_bipolar */
	{-10,10},
	{-1,1},
	{-0.1,0.1},
	{-0.02,0.02},
/* dt282x_ai_hi_unipolar */
	{-10,10},
	{-1,1},
	{-0.1,0.1},
	{-0.02,0.02},
/* ni_E_ai */
	{-10,10},
	{-5,5},
	{-2.5,2.5},
	{-1,1},
	{-0.5,0.5},
	{-0.25,0.25},
	{-0.1,0.1},
	{-0.05,0.05},
	{0,20},
	{0,10},
	{0,5},
	{0,2},
	{0,1},
	{0,0.5},
	{0,0.2},
	{0,0.1},
/* ni_E_ai_limited */
	{-5,5},
	{-2.5,2.5},
	{-0.5,0.5},
	{-0.05,0.05},
	{0,10},
	{0,5},
	{0,1},
	{0,0.1},
/* ni_E_ai_limited14 */
	{-5,5},
	{-2.5,2.5},
	{-1,1},
	{-0.5,0.5},
	{-0.25,0.25},
	{-0.1,0.1},
	{-0.05,0.05},
	{0,10},
	{0,5},
	{0,2},
	{0,1},
	{0,0.5},
	{0,0.2},
	{0,0.1},
/* pcl711b_ai */
	{-5,5},
	{-2.5,2.5},
	{-1.25,1.25},
	{-0.625,0.625},
	{-0.3125,0.3125},
/* acl8112hg_ai */
	{-5,5},
	{-0.5,0.5},
	{-0.05,0.05},
	{-0.005,0.005},
	{0,10},
	{0,1},
	{0,0.1},
	{0,0.01},
	{-10,10},
	{-1,1},
	{-0.1,0.1},
	{-0.01,0.01},
/* acl8112dg_ai */
	{-5,5},
	{-2.5,2.5},
	{-1.25,1.25},
	{-0.625,0.625},
	{0,10},
	{0,5},
	{0,2.5},
	{0,1.25},
	{-10,10},
/* rti800_ai_10_bipolar */
	{-10,10},
	{-1,1},
	{-0.1,0.1},
	{-0.02,0.02},
/* rti800_ai_5_bipolar */
	{-5,5},
	{-0.5,0.5},
	{-0.05,0.05},
	{-0.01,0.01},
/* rti800_ai_10_unipolar */
	{0,10},
	{0,1},
	{0,0.1},
	{0,0.02},
/* das1601_ai_10_bipolar */
	{-10,10},
	{-1,1},
	{-0.1,0.1},
	{-0.01,0.01},
/* das1601_ai_10_unipolar */
	{0,10},
	{0,1},
	{0,0.1},
	{0,0.01},
/* das1602_ai_10_bipolar */
	{-10,10},
	{-5,5},
	{-2.5,2.5},
	{-1.25,1.25},
/* das1602_ai_10_unipolar */
	{0,10},
	{0,5},
	{0,2.5},
	{0,1.25},
/* unknown */
	{0,1},
/* ni_E_ao_ext */
	{-10,10},
	{0,10},
	{-1,1},
	{0,1},
/* ni_E_ao */
	{-10,10},
	{0,10},
/* ---END-RANGE-DEFS--- */
};



comedi_range *comedi_get_ranges(unsigned int range_type)
{
	return comedi_ranges+RANGE_OFFSET(range_type);
}

unsigned int comedi_get_n_ranges(unsigned int range_type)
{
	return RANGE_LENGTH(range_type);
}

double comedi_to_phys(sampl_t data,comedi_range *rng,sampl_t maxdata)
{
	double x;

	if(!rng)return NAN;
	if(!maxdata)return NAN;

	x=data;
	x/=maxdata;
	x*=(rng->max-rng->min);
	x+=rng->min;

	return x;
}

sampl_t comedi_from_phys(double data,comedi_range *rng,sampl_t maxdata)
{
	double s;

	if(!rng)return 0;
	if(!maxdata)return 0;

	s=(data-rng->min)/(rng->max-rng->min)*maxdata;
	if(s<0)return 0;
	if(s>maxdata)return maxdata;

	return (sampl_t)(floor(s+0.5));
}

